# This file is assumed to be put in the JDK root folder
case "$(uname -s)" in

   Darwin)
     # No build support for OS X
     echo 'Mac OS X'
     ;;

   Linux)
     # Download freetype on demand on *nix systems
     # ./freetype/lib must contain the libraries, ./freetype/include must contain the headers
     freetype="./freetype"
     rm -rf $freetype
     
     # Following link was broken on 04 May 2018:
     #wget -O freetype.zip http://mirror.netcologne.de/savannah/freetype/ft271.zip
     # Let's try this one again:
     wget -O freetype.zip http://download.savannah.gnu.org/releases/freetype/ft271.zip

     unzip freetype.zip -d $freetype && f="$freetype"/* && mv "$freetype"/*/* "$freetype" && rmdir "${f[@]}"
     ln -s /usr/lib/x86_64-linux-gnu $freetype/lib
     ;;

   CYGWIN*|MINGW32*|MSYS*)
     # MS machines must be configured with cygwin and have to store freetype in C:\freetype\
     # this has to include a two folders: "include" and "lib"
     echo 'MS Windows'
     freetype="/cygdrive/c/freetype/"
     ;;
esac

type="$1"
version="$2"

if [ "$(expr substr $(uname -s) 1 5)" == "Linux" ]; then
	# export these only on linux, other compilers may crash with unknown options ...
	# export CFLAGS="$CFLAGS -Wno-deprecated-declarations"
	# export CXXFLAGS="$CXXFLAGS -Wno-deprecated-declarations"
  export CFLAGS="-w"
  export CXXFLAGS="-w"
fi


echo "configure $type" &&
bash configure --with-freetype="$freetype" --with-debug-level="$type" \
	--with-user-release-suffix="AntTracks-$version" \
	--enable-hotspot-test-in-build &&
	
echo "build $type" &&
make CONF="$type" DEBUG_BINARIES=true DISABLE_HOTSPOT_OS_VERSION_CHECK=true images 2>&1 &&

#echo "package $type" &&
#find build/*/images/*-image -type f -iname "*.jar" -exec bash ./rezip.sh {} \; &&
#find build/*/images/*-image -type f -iname "*.diz" -exec bash ./rezip.sh {} \; &&
#find build/*/images/*-image -type f -iname "*.zip" -exec bash ./rezip.sh {} \; &&

echo "Fix JavaFX for $type" &&
bash build_AntTracksVM_type_FixJavaFX.sh

echo "finished $type"

result=$?

#if [ $result -ne 0 ]; then
#	lc=$(cat "$log_file" | grep -e "internal compiler error" -e "differs from the endian expected to be found in the target" -e "cannot access NonReadableChannelException" | wc -l)
#	if [ $lc -gt 0 ]; then
#		bash build_AntTracksVM_type.sh "$type" "$version"
#	else
#		exit $result
#	fi
#else
#	exit 0
#fi

exit $result
